#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Data::Filter::Compiler module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 28;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Data::Filter::Compiler';

        use_ok('Tester');
        use_ok('Data::Filter::Traverser');
        use_ok('Data::Filter::Factory');
        use_ok('Mentat::Message::Value::IDEA');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Data::Filter::Compiler::VERSION) if Tester::D();

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new(dt_resolver => Mentat::Message::Value::IDEA->new());
isa_ok($instance, $TESTED_CLASS);

# View the created instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D(2);

# Instantinate the filter factory object
my $ffactory = Data::Filter::Factory->new_from_file('../../conf/filter-grammar.conf');
isa_ok($ffactory, 'Data::Filter::Factory');

#-------------------------------------------------------------------------------
# Create filters by using rules and compile them afterwards
#-------------------------------------------------------------------------------

my @RULES = (
        [
            'Source/IP4 == "195.113.144.194"',
            'Data::Filter::Rule::OperationBinary',
            '(Source/IP4 EQ "195.113.144.194")',
            '(Source/IP4 EQ "195.113.144.194")',
        ],
        [
            'Target/IP4 == "195.113.144.194"',
            'Data::Filter::Rule::OperationBinary',
            '(Target/IP4 EQ "195.113.144.194")',
            '(Target/IP4 EQ "195.113.144.194")',
        ],
        [
            'CreateTime <= "2012-10-09T10:12:06Z"',
            'Data::Filter::Rule::OperationBinary',
            '(CreateTime LE "2012-10-09T10:12:06Z")',
            '(CreateTime LE "2012-10-09 10:12:06Z")',
        ],
        [
            'DetectTime <= "2012-10-09 10:12:06Z"',
            'Data::Filter::Rule::OperationBinary',
            '(DetectTime LE "2012-10-09 10:12:06Z")',
            '(DetectTime LE "2012-10-09 10:12:06Z")',
        ],
        [
            '(2 < 5) AND (4 > 2)',
            'Data::Filter::Rule::OperationBinary',
            '(("2" LT "5") AND ("4" GT "2"))',
            '"1"',
        ],
        [
            '(2 + 10) + ((10 - 2) * 3)',
            'Data::Filter::Rule::OperationBinary',
            '(("2" ADD "10") ADD (("10" SUB "2") MUL "3"))',
            '["36"]',
        ],
        [
            '(Node/Name like ("abc" . "123"))',
            'Data::Filter::Rule::OperationBinary',
            '(Node/Name LIKE ("abc" CON "123"))',
            '(Node/Name LIKE ["(?^:abc123)"])',
        ],
    );

my $counter = 0;
my ($filter, $result);
foreach my $rule (@RULES)
{
    ++$counter;
    $filter = $ffactory->filter_from_string($rule->[0]);
    isa_ok($filter, $rule->[1]);
    is($filter->to_string(), $rule->[2], "Filter test A$counter: $rule->[0]");
    diag("FILTER A$counter: " . Dumper($filter) . "\n") if Tester::D();
    $result = $instance->compile($filter);
    is($result->to_string(), $rule->[3], "Filter test C$counter: $rule->[0]");
    diag("FILTER C$counter: " . Dumper($result) . "\n") if Tester::D();
}
