#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Data::Filter::Traverser module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 240;

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Data::Filter::Traverser';

        use_ok('Tester');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Data::Filter::Traverser::VERSION) if Tester::D();

my $counter;

#-------------------------------------------------------------------------------
# Check the single evaluators
#-------------------------------------------------------------------------------
my @EVAL_RULES_SINGLE = (
        ['not',1,undef,0],
        ['not',0,undef,1],
        ['not','',undef,1],
        ['not','0',undef,1],
        ['not','1',undef,0],
        ['not','a',undef,0],
        ['not',[],undef,0],

        ['and',0,0,0],
        ['and',0,1,0],
        ['and',1,0,0],
        ['and',1,1,1],
        ['and','','',0],
        ['and','','1',0],
        ['and','1','',0],
        ['and','1','1',1],
        ['and','1',1,1],
        ['and',[],[],1],

        ['or',0,0,0],
        ['or',0,1,1],
        ['or',1,0,1],
        ['or',1,1,1],
        ['or','','',0],
        ['or','','1',1],
        ['or','1','',1],
        ['or','1','1',1],
        ['or','1',1,1],
        ['or',[],[],1],

        ['xor',0,0,0],
        ['xor',0,1,1],
        ['xor',1,0,1],
        ['xor',1,1,0],
        ['xor','','',0],
        ['xor','','1',1],
        ['xor','1','',1],
        ['xor','1','1',0],
        ['xor','1',1,0],
        ['xor',[],[],0],

        #-----------------------------------------------------------------------

        ['eq_string',1,2,0],
        ['eq_string',2,1,0],
        ['eq_string',1,1,1],
        ['ne_string',1,2,1],
        ['ne_string',2,1,1],
        ['ne_string',1,1,0],
        ['gt_string',1,2,0],
        ['gt_string',2,1,1],
        ['gt_string',1,1,0],
        ['lt_string',1,2,1],
        ['lt_string',2,1,0],
        ['lt_string',1,1,0],
        ['ge_string',1,2,0],
        ['ge_string',2,1,1],
        ['ge_string',1,1,1],
        ['le_string',1,2,1],
        ['le_string',2,1,0],
        ['le_string',1,1,1],
        ['eq_string',undef,2,0],
        ['eq_string',2,undef,0],
        ['eq_string',undef,undef,0],

        ['eq_numeric',1,2,0],
        ['eq_numeric',2,1,0],
        ['eq_numeric',1,1,1],
        ['ne_numeric',1,2,1],
        ['ne_numeric',2,1,1],
        ['ne_numeric',1,1,0],
        ['gt_numeric',1,2,0],
        ['gt_numeric',2,1,1],
        ['gt_numeric',1,1,0],
        ['lt_numeric',1,2,1],
        ['lt_numeric',2,1,0],
        ['lt_numeric',1,1,0],
        ['ge_numeric',1,2,0],
        ['ge_numeric',2,1,1],
        ['ge_numeric',1,1,1],
        ['le_numeric',1,2,1],
        ['le_numeric',2,1,0],
        ['le_numeric',1,1,1],
        ['eq_numeric',undef,2,0],
        ['eq_numeric',2,undef,0],
        ['eq_numeric',undef,undef,0],

        #-----------------------------------------------------------------------

        ['add',1,1,2],
        ['add','2',3,5],
        ['add',3,'4',7],
        ['add','4','5',9],
        ['add',1.2,1.2,2.4],
        ['add','2.2',3.2,5.4],
        ['add',3.2,'4.2',7.4],
        ['add','4.2','5.2',9.4],

        ['sub',1,1,0],
        ['sub','2',3,-1],
        ['sub',3,'2',1],
        ['sub','4','2',2],
        ['sub',1.5,1.3,0.2],
        ['sub','2.5',3.3,-0.8],
        ['sub',3.5,'2.3',1.2],
        ['sub','4.5','2.3',2.2],

        ['mul',1,2,2],
        ['mul','2',3,6],
        ['mul',3,'4',12],
        ['mul','4','5',20],

        ['div',2,1,2],
        ['div','5',2,2.5],
        ['div',6,'3',2],
        ['div','10','4',2.5],

        ['mod',7,2,1],
        ['mod','9',3,0],
        ['mod',29,'3',2],
        ['mod','15','4',3],

        ['con',12,34,'1234'],
        ['con','ab',12,'ab12'],
        ['con',12,'cd','12cd'],
        ['con','ab','cd','abcd'],

        ['rep',12,4,'12121212'],
        ['rep','ax',3,'axaxax'],
        ['rep',23,'3','232323'],
        ['rep','ef','3','efefef'],
    );

$counter = 0;
foreach my $rule (@EVAL_RULES_SINGLE)
{
    ++$counter;
    is($TESTED_CLASS->evaluate(@{$rule}[0..2]), $rule->[3], "Single evaluator test $counter: " . Tester::P_C($rule));
}

#-------------------------------------------------------------------------------
# Check the multi evaluators
#-------------------------------------------------------------------------------
my @EVAL_RULES_MULTI = (
        ['not',[0,0,0,1],undef,1],
        ['not',[1,1,1,0],undef,1],
        ['not',[0,0,0,0],undef,1],
        ['not',[1,1,1,1],undef,0],

        ['and',[0,0],[0,0],0],
        ['and',[0,0],[0,1],0],
        ['and',[1,0],[0,0],0],
        ['and',[1,1],[1,1],1],

        ['or',[0,0],[0,0],0],
        ['or',[0,0],[0,1],1],
        ['or',[1,0],[0,0],1],
        ['or',[1,1],[1,1],1],

        ['xor',[0,0],[0,0],0],
        ['xor',[0,0],[0,1],1],
        ['xor',[1,0],[0,0],1],
        ['xor',[1,1],[1,1],0],

        #-----------------------------------------------------------------------

        ['eq_string',[1,2,3,4,5],[1],1],
        ['eq_string',[1],[1,2,3,4,5],1],
        ['eq_string',[1,2,3,4,5],[5,6],1],
        ['eq_string',[5,6],[1,2,3,4,5],1],
        ['eq_string',[1,2,3,4,5],[6,7],0],
        ['eq_string',[6,7],[1,2,3,4,5],0],
        ['ne_string',[1,2,3,4,5],[1],1],
        ['ne_string',[1],[1,2,3,4,5],1],
        ['ne_string',[1,2,3,4,5],[5,6],1],
        ['ne_string',[5,6],[1,2,3,4,5],1],
        ['ne_string',[1,2,3,4,5],[6,7],1],
        ['ne_string',[6,7],[1,2,3,4,5],1],
        ['gt_string',[1,2,3,4,5],[1],1],
        ['gt_string',[1],[1,2,3,4,5],0],
        ['gt_string',[1,2,3,4,5],[5,6],0],
        ['gt_string',[5,6],[1,2,3,4,5],1],
        ['gt_string',[1,2,3,4,5],[6,7],0],
        ['gt_string',[6,7],[1,2,3,4,5],1],
        ['lt_string',[1,2,3,4,5],[1],0],
        ['lt_string',[1],[1,2,3,4,5],1],
        ['lt_string',[1,2,3,4,5],[5,6],1],
        ['lt_string',[5,6],[1,2,3,4,5],0],
        ['lt_string',[1,2,3,4,5],[6,7],1],
        ['lt_string',[6,7],[1,2,3,4,5],0],
        ['ge_string',[1,2,3,4,5],[1],1],
        ['ge_string',[1],[1,2,3,4,5],1],
        ['ge_string',[1,2,3,4,5],[5,6],1],
        ['ge_string',[5,6],[1,2,3,4,5],1],
        ['ge_string',[1,2,3,4,5],[6,7],0],
        ['ge_string',[6,7],[1,2,3,4,5],1],
        ['le_string',[1,2,3,4,5],[1],1],
        ['le_string',[1],[1,2,3,4,5],1],
        ['le_string',[1,2,3,4,5],[5,6],1],
        ['le_string',[5,6],[1,2,3,4,5],1],
        ['le_string',[1,2,3,4,5],[6,7],1],
        ['le_string',[6,7],[1,2,3,4,5],0],

        ['eq_numeric',[1,2,3,4,5],[1],1],
        ['eq_numeric',[1],[1,2,3,4,5],1],
        ['eq_numeric',[1,2,3,4,5],[5,6],1],
        ['eq_numeric',[5,6],[1,2,3,4,5],1],
        ['eq_numeric',[1,2,3,4,5],[6,7],0],
        ['eq_numeric',[6,7],[1,2,3,4,5],0],
        ['ne_numeric',[1,2,3,4,5],[1],1],
        ['ne_numeric',[1],[1,2,3,4,5],1],
        ['ne_numeric',[1,2,3,4,5],[5,6],1],
        ['ne_numeric',[5,6],[1,2,3,4,5],1],
        ['ne_numeric',[1,2,3,4,5],[6,7],1],
        ['ne_numeric',[6,7],[1,2,3,4,5],1],
        ['gt_numeric',[1,2,3,4,5],[1],1],
        ['gt_numeric',[1],[1,2,3,4,5],0],
        ['gt_numeric',[1,2,3,4,5],[5,6],0],
        ['gt_numeric',[5,6],[1,2,3,4,5],1],
        ['gt_numeric',[1,2,3,4,5],[6,7],0],
        ['gt_numeric',[6,7],[1,2,3,4,5],1],
        ['lt_numeric',[1,2,3,4,5],[1],0],
        ['lt_numeric',[1],[1,2,3,4,5],1],
        ['lt_numeric',[1,2,3,4,5],[5,6],1],
        ['lt_numeric',[5,6],[1,2,3,4,5],0],
        ['lt_numeric',[1,2,3,4,5],[6,7],1],
        ['lt_numeric',[6,7],[1,2,3,4,5],0],
        ['ge_numeric',[1,2,3,4,5],[1],1],
        ['ge_numeric',[1],[1,2,3,4,5],1],
        ['ge_numeric',[1,2,3,4,5],[5,6],1],
        ['ge_numeric',[5,6],[1,2,3,4,5],1],
        ['ge_numeric',[1,2,3,4,5],[6,7],0],
        ['ge_numeric',[6,7],[1,2,3,4,5],1],
        ['le_numeric',[1,2,3,4,5],[1],1],
        ['le_numeric',[1],[1,2,3,4,5],1],
        ['le_numeric',[1,2,3,4,5],[5,6],1],
        ['le_numeric',[5,6],[1,2,3,4,5],1],
        ['le_numeric',[1,2,3,4,5],[6,7],1],
        ['le_numeric',[6,7],[1,2,3,4,5],0],

        #-----------------------------------------------------------------------

        ['add',[1],[1,2,3],[2,3,4]],
        ['add',[1,2,3],[2],[3,4,5]],
        ['add',[1,2,3],[1,2,3],[2,4,6]],
        ['add',[1,2,3],[],[]],
        ['add',[],[1,2,3],[]],

        ['sub',[6],[1,2,3],[5,4,3]],
        ['sub',[6,5,4],[2],[4,3,2]],
        ['sub',[4,5,6],[1,2,3],[3,3,3]],
        ['sub',[1,2,3],[],[]],
        ['sub',[],[1,2,3],[]],

        ['mul',[2,3,4],[2],[4,6,8]],
        ['mul',[2],[2,3,4],[4,6,8]],
        ['mul',[1,2,3],[2,3,4],[2,6,12]],
        ['mul',[1,2,3],[],[]],
        ['mul',[],[1,2,3],[]],

        ['div',[10,9,8],[2],[5,4.5,4]],
        ['div',[10],[2,4,5],[5,2.5,2]],
        ['div',[10,9,8],[2,3,4],[5,3,2]],
        ['div',[1,2,3],[],[]],
        ['div',[],[1,2,3],[]],

        ['mod',[10,9,7],[4],[2,1,3]],
        ['mod',[10],[6,4,3],[4,2,1]],
        ['mod',[10,9,7],[4,3,2],[2,0,1]],
        ['mod',[1,2,3],[],[]],
        ['mod',[],[1,2,3],[]],

        ['con',[1,2,3],[1],['11','21','31']],
        ['con',[1],['a','b','c'],['1a','1b','1c']],
        ['con',['a','b','c'],['a','b','c'],['aa','bb','cc']],
        ['con',['a','b','c'],[],[]],
        ['con',[],['a','b','c'],[]],

        ['rep',['a','b','c'],[3],['aaa','bbb','ccc']],
        ['rep',['x'],[2,3,4],['xx','xxx','xxxx']],
        ['rep',['a','b','c'],[2,3,'4'],['aa','bbb','cccc']],
        ['rep',['a','b','c'],[],[]],
        ['rep',[],['a','b','c'],[]],
    );

$counter = 0;
foreach my $rule (@EVAL_RULES_MULTI)
{
    ++$counter;
    is_deeply($TESTED_CLASS->evaluate_n(@{$rule}[0..2]), $rule->[3], "Multi evaluator test $counter: " . Tester::P_C($rule));
}
