#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Mentat::Message::Convertor::Mongo module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 2033;

use Cwd qw(abs_path);
use FindBin;
use lib (abs_path("$FindBin::Bin/../lib") =~ m{^(/.*)$})[0];
use lib (abs_path("$FindBin::Bin/../../../lib") =~ m{^(/.*)$})[0];

# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Mentat::Message::Convertor::Mongo';

        use_ok('Tester');
        use_ok('Mentat::Handyman');
        use_ok('Mentat::Message::Factory');
        use_ok('Mentat::Message::Convertor::IDEA');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
Tester::DEBUG_ON();
#Tester::DEBUG_ON(10);

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Mentat::Message::Convertor::Mongo::VERSION) if Tester::D();
### $Mentat::Message::Convertor::Mongo::VERSION

#-------------------------------------------------------------------------------
# Prepare builder configurations
my $BUILDER_CONFIG = {
        'analyzer_id'       => 'omnius.cesnet.cz',
        'analyzer_os'       => 'linux',
        'analyzer_osver'    => '2.6.35-32-generic',
        'analyzer_host'     => 'somehost.somedomain',
        'analyzer_ipv4'     => '127.0.0.1',
        'analyzer_ipv6'     => '1::1',
    };

# Prepare handyman for creating messages
my $handyman = Mentat::Handyman->new(template_file        => '../../conf/msgtmplts.json',
                                     temp_folder          => '../../spool',
                                     pickup_folder        => '../../spool',
                                     cache_folder         => '../../spool',
                                     #schema_file          => '../../conf/schema-idea.json',
                                     template_schema_file => '../../conf/schema-msgtmplts.json',
                                     user                 => 'mentat',
                                     builder_config       => $BUILDER_CONFIG,
                                     strict               => 1);
isa_ok($handyman, 'Mentat::Handyman');

#-------------------------------------------------------------------------------

# Now attempt to create a class instance and check it
my $instance = Mentat::Message::Convertor::Mongo->new();
isa_ok($instance, "Mentat::Message::Convertor::Mongo");

# View the created instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D();
### $instance

# Now attempt to create a class instance and check it
my $instance_inv = Mentat::Message::Convertor::IDEA->new();
isa_ok($instance_inv, "Mentat::Message::Convertor::IDEA");

# View the created instance
diag("Instance_inv dump 01: " . Dumper($instance_inv)) if Tester::D();
### $instance

#-------------------------------------------------------------------------------

#my @tlist = ();
#my @tlist = ('n6-020');
my @tlist = $handyman->templates();
my ($idea, $tree, $idea_back, $hash, $template, @msg_args, $ipx, $ipy);
foreach my $tid (@tlist) {
    eval {
        diag("TEMPLATE: $tid") if Tester::D();

        # Generate the test message
        $template = $handyman->template($tid);
        @msg_args = $handyman->parse_args($template->{'cmd_example'});
        $idea     = $handyman->generate($tid, {'detect_time' => time}, @msg_args);
        isa_ok($idea, 'Mentat::Message::IDEA');
        diag("IDEA message before: ".$idea->to_string(1)) if Tester::D();
        $ipx = $idea->path_value('Source/IP4');

        $tree = $instance->convert($idea);
        isa_ok($tree, 'Data::Tree');
        diag("Converted message from IDEA: ".Dumper($tree)) if Tester::D();
        $hash = $tree->hash();
        $ipy = $hash->{Source}->[0]->{IP4}->[0]->{ip}->data;
        $ipy = Value::Convertor->bin_to_ipv4str($ipy);

        is("$ipx", "$ipy", "IP Check test A:'$tid':'$ipx'");

        $idea_back = $instance_inv->convert($tree, 'mongo');
        isa_ok($idea_back, 'Mentat::Message::IDEA');
        diag("Converted message from MONGO: ".$idea_back->to_string(1)) if Tester::D();
        
        my ($src_mpath, $value) = $idea_back->iterate();
        while ($src_mpath) {
            is($value, $idea->path_value($src_mpath));

            # Continue iteration
            ($src_mpath, $value) = $idea_back->iterate();
        }
    };
    if ($@) {
        next unless $tid;
        fail("Message generation test for template '$tid': $@");
    }
}
