#!/usr/bin/perl -T

use strict;
use warnings;

#*******************************************************************************
# Whois::Dump module/class tests
#
# Author:   Jan Mach, jan.mach@cesnet.cz
# Version:  0.1
# Purpose:
#
#*******************************************************************************

use Data::Dumper;
#use Smart::Comments;

use Test::More tests => 17;
# First test if the module correctly compiles
BEGIN {
        use vars qw($TESTED_CLASS);
        $TESTED_CLASS = 'Whois::Dump';

        use_ok('Tester');
        use_ok('Value::Convertor');
        use_ok('Value::IPv4ADDR');
        use_ok($TESTED_CLASS);
    }

# Debug output explicit ON switch
#Tester::DEBUG_ON();

# Debug output explicit OFF switch
#Tester::DEBUG_OFF();

# User should know, which version is beeing tested (just for sure)
diag("$TESTED_CLASS version: " . $Whois::Dump::VERSION) if Tester::D();

# Now attempt to create a class instance and check it
my $instance = $TESTED_CLASS->new('./data/whois.dump', 2);
isa_ok($instance, $TESTED_CLASS);

# View the created instance
diag("Instance dump 01: " . Dumper($instance)) if Tester::D();

my @resultsA = $instance->find('146.102.1.1');
is(scalar @resultsA,                        2,                          "Count check 01");
is(scalar $resultsA[0]->{'netname'},        'VSE-TCZ',                  "Value check 01");
is(scalar $resultsA[0]->{'inetnum_int'},    '2456158208:2456223743',    "Value check 02");

my @resultsB = $instance->find('147.228.2.2');
is(scalar @resultsB,                        2,                          "Count check 02");
is(scalar $resultsB[0]->{'netname'},        'ZCU-TCZ',                  "Value check 03");
is(scalar $resultsB[0]->{'inetnum_int'},    '2481192960:2481258495',    "Value check 04");

my @resultsC = $instance->find('195.113.215.100');
is(scalar @resultsC,                        3,                          "Count check 03");
is(scalar $resultsC[0]->{'netname'},        'MENDELU-TCZ',              "Value check 05");
is(scalar $resultsC[0]->{'inetnum_int'},    '3279017728:3279018751',    "Value check 06");

is($instance->find_abuse('146.102.1.1'),     'abuse@vse.cz',     "Abuse check 01");
is($instance->find_abuse('147.228.2.2'),     'abuse@zcu.cz',     "Abuse check 02");
is($instance->find_abuse('195.113.215.100'), 'abuse@mendelu.cz', "Abuse check 03");
